@@ -24,26 +24,21 @@
 
 #include <sys/time.h>
 #include <time.h>
+#include <string.h>
 
 #include <glib/glist.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkmessagedialog.h>
-#include <gtk/gtkprogressbar.h>
-#include <gtk/gtkstatusbar.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkstatusbar.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktreemodel.h>
+#include <libgnomevfs/gnome-vfs-volume.h>
+#include <libgnomevfs/gnome-vfs-volume-monitor.h>
+#include <gtk/gtk.h>
 
 #include "sj-error.h"
 #include "sj-extracting.h"
 #include "sj-util.h"
 #include "sj-play.h"
-
+#include "sj-inhibit.h"
+#include "sj-genres.h"
 
 typedef struct {
   int seconds;
@@ -113,6 +108,11 @@ static int total_duration;
  */
 static Progress before;
 
+/**
+* The cookie returned from PowerManagement
+*/
+static guint cookie;
+
 /**
  * Build the absolute filename for the specified track.
  *
@@ -173,6 +173,8 @@ cleanup (void)
 
   nautilus_burn_drive_unlock (drive);
 
+  sj_uninhibit (cookie);
+
   /* Remove any state icons from the model */
   if (current.stamp) {
     /* TODO: has to be a better way to do that test */
@@ -532,58 +534,35 @@ on_main_window_focus_in (GtkWidget * widget, GdkEventFocus * event, gpointer dat
 }
 
 /**
- * Show the "Finished!" dialog, allowing the user to open Nautilus if he wants.
+ * Handle any post-rip actions
  */
 static void
-show_finished_dialog (const TrackDetails *track)
+finished_actions (void)
 {
-  GtkWidget *dialog;
-  int result;
-  char *base;
-
-  base = NULL;
-  /* Find the deepest common directory. */
-  g_list_foreach (paths, (GFunc)base_finder, &base);
-
-  cleanup ();
-
-  char *message;
-  message = g_strdup_printf (_("%s has been copied successfully."), track->album->title);
-
-  dialog = gtk_message_dialog_new (GTK_WINDOW (main_window),
-                                   GTK_DIALOG_DESTROY_WITH_PARENT,
-                                   GTK_MESSAGE_INFO,
-                                   GTK_BUTTONS_NONE,
-                                   (message));
-
-  g_free (message);
-  /* If we eject when finished, eject now, otherwise add a button */
-  if (eject_finished) {
-    nautilus_burn_drive_eject (drive);
-  } else {
-    gtk_dialog_add_buttons (GTK_DIALOG (dialog), _("_Eject"), 2, NULL);
-  }
-
   /* Trigger glowing effect after copy */
-  g_signal_connect (G_OBJECT (dialog), "focus-in-event",  G_CALLBACK (on_main_window_focus_in),  NULL);
+  g_signal_connect (G_OBJECT (main_window), "focus-in-event",
+                    G_CALLBACK (on_main_window_focus_in),  NULL);
   gtk_window_set_urgency_hint (GTK_WINDOW (main_window), TRUE);
 
-  gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-                          GTK_STOCK_OPEN, 1,
-                          GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
-                          NULL);
-  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
-  result = gtk_dialog_run (GTK_DIALOG (dialog));
-  gtk_widget_destroy (dialog);
-  if (result == 1) {
-    char *command;
-    command = g_strdup_printf ("gnome-open \"%s\"", base);
-    g_spawn_command_line_async (command, NULL);
-    g_free (command);
-  } else if (result == 2) {
+  /* Maybe eject */
+  if (eject_finished) {
     nautilus_burn_drive_eject (drive);
   }
-  g_free (base);
+  
+  /* Maybe open the target directory */
+  if (open_finished) {
+    char *base, *command;
+
+    base = NULL;
+    /* Find the deepest common directory. */
+    g_list_foreach (paths, (GFunc)base_finder, &base);
+    
+    command = g_strdup_printf ("gnome-open \"%s\"", base);
+    g_spawn_command_line_async (command, NULL);
+    
+    g_free (command);
+    g_free (base);
+  }
 }
 
 /**
@@ -612,8 +591,12 @@ on_completion_cb (SjExtractor *extractor, gpointer data)
     /* And go and do it all again */
     pop_and_extract ((int*)data);
   } else {
-    show_finished_dialog (track);
-    
+    /* If we got here then the track state has been set to IDLE already, so
+       unset the current iterator */
+    current.stamp = 0;
+    finished_actions ();
+    cleanup ();
+
     if (autostart) {
       gtk_main_quit ();
     }
@@ -742,6 +725,12 @@ on_extract_activate (GtkWidget *button, gpointer user_data)
     g_warning ("Could not lock drive: %s", reason);
     g_free (reason);
   }
+  
+  cookie = sj_inhibit (g_get_application_name (),
+                       _("Extracting audio from CD"));
+
+  /* Save the genre */
+  save_genre (genre_entry);
 
   /* Start the extracting */
   extracting = TRUE;
@@ -766,7 +755,7 @@ on_extract_activate (GtkWidget *button, gpointer user_data)
  * string.
  */
 static char*
-sanitize_path (const char* str)
+sanitize_path (const char* str, const char* filesystem_type)
 {
   gchar *res = NULL;
   gchar *s;
@@ -778,6 +767,15 @@ sanitize_path (const char* str)
   s = g_strdup(str);
   /* Replace path seperators with a hyphen */
   g_strdelimit (s, "/", '-');
+  
+  /* filesystem specific sanitizing */
+  if (filesystem_type) {
+    if ((strcmp (filesystem_type, "vfat") == 0) ||
+        (strcmp (filesystem_type, "ntfs") == 0)) {
+      g_strdelimit (s, "\\:*?\"<>|", ' ');
+    }
+  }
+    
   if (strip_chars) {
     /* Replace separators with a hyphen */
     g_strdelimit (s, "\\:|", '-');
@@ -815,11 +813,23 @@ filepath_parse_pattern (const char* pattern, const TrackDetails *track)
 {
   /* p is the pattern iterator, i is a general purpose iterator */
   const char *p;
-  char *tmp, *str;
+  char *tmp, *str, *base_path, *filesystem_type = NULL;
   GString *s;
+  GnomeVFSVolumeMonitor *monitor;
+  GnomeVFSVolume *volume;
 
   if (pattern == NULL || pattern[0] == 0)
     return g_strdup (" ");
+    
+  if ((base_path = gnome_vfs_get_local_path_from_uri (base_uri))) {
+    monitor = gnome_vfs_get_volume_monitor ();
+    
+    if ((volume = gnome_vfs_volume_monitor_get_volume_for_path (monitor, base_path))) {
+      filesystem_type = gnome_vfs_volume_get_filesystem_type (volume);
+      gnome_vfs_volume_unref (volume);
+    }
+    g_free (base_path);
+  }
 
   s = g_string_new (NULL);
 
@@ -854,27 +864,27 @@ filepath_parse_pattern (const char* pattern, const TrackDetails *track)
        */
       switch (*++p) {
       case 't':
-        string = sanitize_path (track->album->title);
+        string = sanitize_path (track->album->title, filesystem_type);
         break;
       case 'T':
         tmp = g_utf8_strdown (track->album->title, -1);
-        string = sanitize_path (tmp);
+        string = sanitize_path (tmp, filesystem_type);
         g_free (tmp);
         break;
       case 'a':
-        string = sanitize_path (track->album->artist);
+        string = sanitize_path (track->album->artist, filesystem_type);
         break;
       case 'A':
         tmp = g_utf8_strdown (track->album->artist, -1);
-        string = sanitize_path (tmp);
+        string = sanitize_path (tmp, filesystem_type);
         g_free (tmp);
         break;
       case 's':
-        string = sanitize_path (track->album->artist_sortname ? track->album->artist_sortname : track->album->artist);
+        string = sanitize_path (track->album->artist_sortname ? track->album->artist_sortname : track->album->artist, filesystem_type);
         break;
       case 'S':
         tmp = g_utf8_strdown (track->album->artist_sortname ? track->album->artist_sortname : track->album->artist, -1);
-        string = sanitize_path (tmp);
+        string = sanitize_path (tmp, filesystem_type);
         g_free(tmp);
         break;
       default:
@@ -893,27 +903,27 @@ filepath_parse_pattern (const char* pattern, const TrackDetails *track)
        */
       switch (*++p) {
       case 't':
-        string = sanitize_path (track->title);
+        string = sanitize_path (track->title, filesystem_type);
         break;
       case 'T':
         tmp = g_utf8_strdown (track->title, -1);
-        string = sanitize_path (tmp);
+        string = sanitize_path (tmp, filesystem_type);
         g_free(tmp);
         break;
       case 'a':
-        string = sanitize_path (track->artist);
+        string = sanitize_path (track->artist, filesystem_type);
         break;
       case 'A':
         tmp = g_utf8_strdown (track->artist, -1);
-        string = sanitize_path (tmp);
+        string = sanitize_path (tmp, filesystem_type);
         g_free(tmp);
         break;
       case 's':
-        string = sanitize_path (track->artist_sortname ? track->album->artist_sortname : track->artist);
+        string = sanitize_path (track->artist_sortname ? track->album->artist_sortname : track->artist, filesystem_type);
         break;
       case 'S':
         tmp = g_utf8_strdown (track->artist_sortname ? track->album->artist_sortname : track->artist, -1);
-        string = sanitize_path (tmp);
+        string = sanitize_path (tmp, filesystem_type);
         g_free(tmp);
         break;
       case 'n':
@@ -950,6 +960,8 @@ filepath_parse_pattern (const char* pattern, const TrackDetails *track)
     if (go_next)
       ++p;
   }
+  
+  g_free (filesystem_type); 
 
   str = s->str;
   g_string_free (s, FALSE);
