@@ -52,6 +52,7 @@
 #include "sj-main.h"
 #include "sj-prefs.h"
 #include "sj-play.h"
+#include "sj-genres.h"
 #include "gedit-message-area.h"
 #include "bacon-volume.h"
 
@@ -89,6 +90,7 @@ char *base_uri;
 NautilusBurnDrive *drive = NULL;
 gboolean strip_chars;
 gboolean eject_finished;
+gboolean open_finished;
 gboolean extracting = FALSE;
 
 static gint total_no_of_tracks;
@@ -376,7 +378,7 @@ set_message_area_text_and_icon (GeditMessageArea *message_area,
   gtk_widget_show (vbox);
   gtk_box_pack_start (GTK_BOX (hbox_content), vbox, TRUE, TRUE, 0);
 
-  primary_markup = g_strdup_printf ("<b>%s</b>", primary_text);
+  primary_markup = g_markup_printf_escaped ("<b>%s</b>", primary_text);
   primary_label = gtk_label_new (primary_markup);
   g_free (primary_markup);
   gtk_widget_show (primary_label);
@@ -386,8 +388,8 @@ set_message_area_text_and_icon (GeditMessageArea *message_area,
   gtk_misc_set_alignment (GTK_MISC (primary_label), 0, 0.5);
 
   if (secondary_text != NULL) {
-    secondary_markup = g_strdup_printf ("<small>%s</small>",
-					secondary_text);
+    secondary_markup = g_markup_printf_escaped ("<small>%s</small>",
+						secondary_text);
     secondary_label = gtk_label_new (secondary_markup);
     g_free (secondary_markup);
     gtk_widget_show (secondary_label);
@@ -424,7 +426,7 @@ set_message_area (GtkWidget *container,
 		      0);
 
   g_object_add_weak_pointer (G_OBJECT (current_message_area),
-			     (gpointer *)&current_message_area);
+			     (gpointer)&current_message_area);
 }
 
 static GtkWidget* musicbrainz_submit_message_area_new (char *title,
@@ -683,7 +685,7 @@ AlbumDetails* multiple_album_dialog(GList *albums)
 /**
  * The GConf key for the base path changed
  */
-void baseuri_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void baseuri_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_BASEURI) == 0);
   g_free (base_uri);
@@ -698,7 +700,7 @@ void baseuri_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry,
 /**
  * The GConf key for the directory pattern changed
  */
-void path_pattern_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void path_pattern_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_PATH_PATTERN) == 0);
   if (entry->value == NULL) {
@@ -713,7 +715,7 @@ void path_pattern_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *en
 /**
  * The GConf key for the filename pattern changed
  */
-void file_pattern_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void file_pattern_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_FILE_PATTERN) == 0);
   if (entry->value == NULL) {
@@ -728,7 +730,7 @@ void file_pattern_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *en
 /**
  * The GConf key for the paranoia mode has changed
  */
-void paranoia_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void paranoia_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_PARANOIA) == 0);
   if (entry->value == NULL) {
@@ -744,7 +746,7 @@ void paranoia_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry,
 /**
  * The GConf key for the strip characters option changed
  */
-void strip_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void strip_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_STRIP) == 0);
   if (entry->value == NULL) {
@@ -757,7 +759,7 @@ void strip_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gp
 /**
  * The GConf key for the eject when finished option changed
  */
-void eject_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void eject_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_EJECT) == 0);
   if (entry->value == NULL) {
@@ -767,10 +769,23 @@ void eject_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gp
   }
 }
 
+/**
+ * The GConf key for the eject when finished option changed
+ */
+static void open_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+{
+  g_assert (strcmp (entry->key, GCONF_OPEN) == 0);
+  if (entry->value == NULL) {
+    open_finished = FALSE;
+  } else {
+    open_finished = gconf_value_get_bool (entry->value);
+  }
+}
+
 /**
  * The GConf key for audio volume changes
  */
-void audio_volume_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void audio_volume_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_AUDIO_VOLUME) == 0);
   
@@ -1085,7 +1100,7 @@ prefs_get_default_device (void)
 /**
  * The GConf key for the device changed
  */
-void device_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void device_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   const char *device;
   gboolean ignore_no_media = GPOINTER_TO_INT (user_data);
@@ -1116,7 +1131,7 @@ void device_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, g
   set_device (device, ignore_no_media);
 }
 
-void profile_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void profile_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   GMAudioProfile *profile;
   
@@ -1151,7 +1166,7 @@ void profile_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry,
 /**
  * Configure the http proxy
  */
-void
+static void
 http_proxy_setup (GConfClient *client)
 {
   if (!gconf_client_get_bool (client, GCONF_HTTP_PROXY_ENABLE, NULL)) {
@@ -1171,7 +1186,7 @@ http_proxy_setup (GConfClient *client)
 /**
  * The GConf key for the HTTP proxy being enabled changed.
  */
-void http_proxy_enable_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void http_proxy_enable_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_HTTP_PROXY_ENABLE) == 0);
   if (entry->value == NULL) return;
@@ -1181,7 +1196,7 @@ void http_proxy_enable_changed_cb (GConfClient *client, guint cnxn_id, GConfEntr
 /**
  * The GConf key for the HTTP proxy changed.
  */
-void http_proxy_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void http_proxy_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_HTTP_PROXY) == 0);
   if (entry->value == NULL) return;
@@ -1191,7 +1206,7 @@ void http_proxy_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entr
 /**
  * The GConf key for the HTTP proxy port changed.
  */
-void http_proxy_port_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void http_proxy_port_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_HTTP_PROXY_PORT) == 0);
   if (entry->value == NULL) return;
@@ -1399,42 +1414,6 @@ void on_contents_activate(GtkWidget *button, gpointer user_data) {
   }
 }
 
-
-static const char* const genres[] = {
-  "Ambient",
-  "Blues",
-  "Classical",
-  "Country",
-  "Dance",
-  "Electronica",
-  "Folk",
-  "Funk",
-  "Jazz",
-  "Latin",
-  "Pop",
-  "Rap",
-  "Reggae",
-  "Rock",
-  "Soul",
-  "Spoken Word",
-  NULL
-};
-
-static GtkTreeModel* create_genre_list(void) {
-  GtkListStore *store;
-  const char * const *g = genres;
-
-  store = gtk_list_store_new (1, G_TYPE_STRING);
-
-  while (*g != NULL) {
-    GtkTreeIter iter;
-    gtk_list_store_append (store, &iter);
-    gtk_list_store_set (store, &iter, 0, *g++, -1);
-  }
-
-  return GTK_TREE_MODEL (store);
-}
-
 GtkWidget *
 sj_make_volume_button (void)
 {
@@ -1541,6 +1520,7 @@ int main (int argc, char **argv)
   gconf_client_add_dir (gconf_client, GCONF_ROOT, GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
   gconf_client_notify_add (gconf_client, GCONF_DEVICE, device_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_EJECT, eject_changed_cb, NULL, NULL, NULL);
+  gconf_client_notify_add (gconf_client, GCONF_OPEN, open_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_BASEURI, baseuri_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_STRIP, strip_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_AUDIO_PROFILE, profile_changed_cb, NULL, NULL, NULL);
@@ -1609,14 +1589,7 @@ int main (int argc, char **argv)
     gtk_size_group_add_widget (size_group, play_button);
   }
 
-  {
-    GtkEntryCompletion *completion;
-    completion = gtk_entry_completion_new ();
-    gtk_entry_completion_set_model (completion, create_genre_list ());
-    gtk_entry_completion_set_text_column (completion, 0);
-    gtk_entry_completion_set_inline_completion (completion, TRUE);
-    gtk_entry_set_completion (GTK_ENTRY (genre_entry), completion);
-  }
+  setup_genre_entry (genre_entry);
 
   track_store = gtk_list_store_new (COLUMN_TOTAL, G_TYPE_INT, G_TYPE_BOOLEAN, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_POINTER);
   gtk_tree_view_set_model (GTK_TREE_VIEW (track_listview), GTK_TREE_MODEL (track_store));
@@ -1705,6 +1678,7 @@ int main (int argc, char **argv)
   paranoia_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_PARANOIA, NULL, TRUE, NULL), NULL);
   strip_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_STRIP, NULL, TRUE, NULL), NULL);
   eject_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_EJECT, NULL, TRUE, NULL), NULL);
+  open_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_OPEN, NULL, TRUE, NULL), NULL);
   audio_volume_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_AUDIO_VOLUME, NULL, TRUE, NULL), NULL);
   if (device == NULL) {
     device_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_DEVICE, NULL, TRUE, NULL), GINT_TO_POINTER (TRUE));
